@@ -12,17 +12,20 @@ import hashlib
 from operator import itemgetter
 
 import yaml
-import markdown2
+import markdown
 from jinja2 import Template, FileSystemLoader, Environment
 import sqlite3
 import re
 import pickle
 import unidecode
 from htmlmin import minify
-
+from lxml import etree as ET
+from copy import deepcopy
 
 class contentFilters():
     exts = ('.md', '.markdown', '.wordpress', '.html')
+    def __init__(self, markdown_extensions=[]):
+        self.markdown_extensions = markdown_extensions
 
     def toHtml(self, txt, ext='.markdown'):
         if ext == '.markdown' or ext == '.md':
@@ -35,7 +38,7 @@ class contentFilters():
             return txt
 
     def markdownToHtml(self, txt):
-        ret = markdown2.markdown(txt)
+        ret = markdown.markdown(txt, extensions=self.markdown_extensions, output_format='xhtml5')
         return ret
 
     def wordpressToHtml(self, txt):
@@ -43,10 +46,71 @@ class contentFilters():
         paras = re.split('\n{2,}', txt)
         paras = ['<p>%s</p>' % p.replace('\n', '<br />') for p in paras]
         txt2 = '\n'.join(paras)
-        return txt2.decode('utf-8')
+        return txt2
 
     def html2Html(self, txt):
-        return txt.decode('utf-8')
+        return txt
+
+def fb_instant_articles_markup(html, url=''):
+    html = "<div>" + html +"</div>"
+    root = ET.fromstring(html)
+    for e in root.iterfind('.//center'):
+        if e.text.replace(' ','') == '***':
+            e.text = ''
+    ET.strip_tags(root, 'center')
+
+    for e in root.findall('.//img'):
+        parent_e = next(e.iterancestors())
+        if parent_e.tag != 'figure':
+            tmp_e = ET.Element('figure')
+            tmp_e.append(deepcopy(e))
+            parent_e.replace(e, tmp_e)
+        
+    for h in root.findall('.//h2'):
+        h.tag='h1'
+    for h in root.findall('.//h3'):
+        h.tag='h2'
+    
+    for e in root.findall('.//p'):
+        elements = [el for el in e]
+        if len(elements)==1 and elements[0].tail==None and e.text==None:
+            tmp_e = deepcopy(elements[0])    
+            parent_e = e.getparent()
+            parent_e.replace(e, tmp_e)    
+        if len(e) == 0 and e.text.strip() == '' and e.tail.strip()=='':
+            e.getparent().remove(e)
+
+        for e in root.findall('./pre/code'):
+            parent_e = e.getparent()
+            tmp_e = ET.Element('blockquote')
+            tmp_a = ET.Element('a')
+            tmp_a.attrib['href']=url
+            tmp_a.text = """
+            Visit the original post on my blog to get the code that was displayed here.
+            """
+            tmp_a.tail=' (Code removed due to limitations of Facebook instant articles.)'
+            tmp_e.append(tmp_a)
+            parent_e.replace(e, tmp_e)
+
+    ret = ET.tostring(root)
+    return ret
+
+def jinja_filter_gravatar(email, size=100, rating='g', default='retro', force_default=False,
+    force_lower=False, use_ssl=False):
+    # source: https://gist.github.com/Alquimista/3499097
+    if use_ssl:
+        url = "https://secure.gravatar.com/avatar/"
+    else:
+        url = "http://www.gravatar.com/avatar/"
+    if force_lower:
+        email = email.lower()
+    hashemail = hashlib.md5(email).hexdigest()
+    link = "{url}{hashemail}?s={size}&d={default}&r={rating}".format(
+        url=url, hashemail=hashemail, size=size,
+        default=default, rating=rating)
+    if force_default:
+        link = link + "&f=y"
+    return link
 
 class Bucket3():
 
@@ -54,11 +118,9 @@ class Bucket3():
         self.verbose = verbose
 
         time.tzset()
-        self.filters = contentFilters()
 
         self.root_url = conf['blog']['url']
         self.root_dir = conf['root_dir']
-        self.mentions_dir = conf['mentions_dir']
         self.data_dir = os.path.join(self.root_dir, '.bucket3', 'data')
         self.posts_dir = os.path.join(self.root_dir, 'posts')
         self.html_dir = os.path.join(self.root_dir, 'html')
@@ -81,11 +143,20 @@ class Bucket3():
         if not os.path.exists(self.data_dir):
             os.makedirs(self.data_dir)
 
+        if 'markdown_extensions' in conf:
+            self.markdown_extensions = conf['markdown_extensions']
+        else:
+            self.markdown_extensions = []
+        self.filters = contentFilters(markdown_extensions=self.markdown_extensions)
+
         blog = conf['blog']
 
         # we will need rss_tags both in templates and program flow,
         # setting both makes it easier.
 
+        if 'posts_prefix' in conf:
+                self.posts_prefix = conf['posts_prefix']
+                
         if 'rss_tags' in conf and conf['rss_tags']:
             self.rss_tags = conf['rss_tags']
             blog['rss_tags'] = conf['rss_tags']
@@ -93,25 +164,16 @@ class Bucket3():
             self.rss_tags = False
             blog['rss_tags'] = False
 
-        if 'theme' in conf and conf['theme']:
-            self.theme = conf['theme']
-        else:
-            self.theme = 'bucket3'
-
         if 'minify_html' in conf:
             self.minify_html = conf['minify_html']
         else:
             self.minify_html = False
 
-        self.template_dir = [
-                os.path.join(self.root_dir, '.bucket3', 'themes', self.theme, 'templates'),
-                os.path.join(self.root_dir, '.bucket3', 'themes', 'bucket3', 'templates'),
-                os.path.join(self.root_dir, 'static'),
-                os.path.join(os.path.dirname(os.path.abspath(__file__)),
-                    '_themes', 'bucket3', 'templates'),  # last resort, the bucket3 theme downloaded with the app
-                ]
+        self.template_dir = [ os.path.join(self.root_dir, 'templates'), ]
 
         self.tpl_env = Environment(loader=FileSystemLoader(self.template_dir))
+        self.tpl_env.filters['gravatar'] = jinja_filter_gravatar
+        self.tpl_env.filters['fbia'] = fb_instant_articles_markup
         self.tpl_env.globals['blog'] = blog
         self.tpl_env.globals['_months'] = [calendar.month_name[i] for i in range(0, 13)]  # yes, needs to start from zero.
         self.tpl_env.globals['_months_short'] = [calendar.month_abbr[i] for i in range(0, 13)]  # yes, needs to start from zero.
@@ -149,11 +211,11 @@ class Bucket3():
         if not os.path.exists(file_dir):
                 os.makedirs(file_dir)
         f = open(os.path.join(file_dir, file_name), 'w')
-        f.write(file_content.encode('utf8'))
+        f.write(file_content)
         f.close()
 
     def util_parse_frontmatter(self, txt):
-        meta = yaml.load(txt)
+        meta = yaml.load(txt, Loader=yaml.FullLoader)
         meta['title'] = meta['title'].strip()
         if type(meta['date']) != datetime:
             meta['date'] = datetime.strptime(meta['date'][:16], '%Y-%m-%d %H:%M')
@@ -200,19 +262,19 @@ class Bucket3():
 
     def fs_post_get(self, path):
         if self.verbose > 1:
-            print "DEBUG: fs_post_get(%s)" % path
+            print("DEBUG: fs_post_get(%s)" % path)
         if not os.path.splitext(path)[1] in self.filters.exts:
             return None
 
         abs_path = self.util_abs_path(path)
 
-        post_id = hashlib.sha1(path).hexdigest()
+        post_id = hashlib.sha1(path.encode('utf-8')).hexdigest()
 
         txt = open(abs_path, 'r').read()
         (dummy, frontmatter, body) = txt.split('---', 2)
 
         if self.verbose > 1:
-            print 'Reading %s...' % path
+            print('Reading %s...' % path)
 
         meta = self.util_parse_frontmatter(frontmatter)
 
@@ -252,15 +314,15 @@ class Bucket3():
 
     def db_post_del(self, id):
         if self.verbose:
-            print "Deleting post [id:%s]..." % id,
+            print("Deleting post [id:%s]..." % id, end='')
         post = self.db_post_get(id)
         if not post and self.verbose:
-            print "Not found. Ignoring."
+            print("Not found. Ignoring.")
             return
         self.db_conn.execute('DELETE FROM posts WHERE id=?', (id,))
         self.rq_post_deps(post)
         if self.verbose:
-            print "Done."
+            print("Done.")
 
     def db_init(self):
         self.db_conn.executescript("""
@@ -283,7 +345,7 @@ class Bucket3():
         p['year'] = dt.year
         p['month'] = '{:02d}'.format(dt.month)
         p['day'] = '{:02d}'.format(dt.day)
-        p['meta'] = pickle.loads(str(row['meta']))
+        p['meta'] = pickle.loads(row['meta'])
         return p
 
     def db_post_get(self, id):
@@ -384,77 +446,58 @@ class Bucket3():
         for task in self.render_Q:
             if task[0] == 'post':
                 if self.verbose:
-                    print 'Rendering post    [id:%s]...' % task[1],
+                    print('Rendering post    [id:%s]...' % task[1], end='')
                 self.render_post(task[1])
                 if self.verbose:
-                    print "Done."
+                    print("Done.")
 
             elif task[0] == 'archive_main':
                 if self.verbose:
-                    print "Rendering archive [main]...",
+                    print("Rendering archive [main]...", end='')
                 self.render_archive_main()
                 if self.verbose:
-                    print "Done."
+                    print("Done.")
 
             elif task[0] == 'archive_year':
                 if self.verbose:
-                    print "Rendering archive [year:%s]..." % task[1],
+                    print("Rendering archive [year:%s]..." % task[1], end='')
                 self.render_archive_year(year=task[1])
                 if self.verbose:
-                    print "Done."
+                    print("Done.")
 
             elif task[0] == 'archive_month':
                 if self.verbose:
-                    print "Rendering archive [year:%s, month:%s]..." % (task[1][0], task[1][1]),
+                    print("Rendering archive [year:%s, month:%s]..." % (task[1][0], task[1][1]), end='')
                 self.render_archive_month(year=task[1][0], month=task[1][1])
                 if self.verbose:
-                    print "Done."
+                    print("Done.")
 
             elif task[0] == 'tag':
                 if self.verbose:
-                    print "Rendering archive [tag:%s]..." % task[1],
+                    print("Rendering archive [tag:%s]..." % task[1], end='')
                 self.render_archive_tag(task[1])
                 if self.verbose:
-                    print "Done."
+                    print("Done.")
 
             elif task[0] == 'rss':
                 self.render_rss()
 
             elif task[0] == 'sitemap':
                 if self.verbose:
-                    print "Rendering sitemap.xml... ",
+                    print("Rendering sitemap.xml... ", end='')
                 self.render_xml_sitemap()
                 if self.verbose:
-                    print "Done."
+                    print("Done.")
 
             elif task[0] == 'homepage':
                 if self.verbose:
-                    print "Rendering homepage...",
+                    print("Rendering homepage...", end='')
                 self.render_home()
                 if self.verbose:
-                    print "Done."
+                    print("Done.")
             # MORE HERE!!!
 
-    def render_html_skel(self):
-        if self.verbose:
-            print "   Populating /_/...",
-        copy_tree(
-            os.path.join(self.root_dir, '.bucket3', 'themes', self.theme, 'assets'),
-            os.path.join(self.html_dir, '_')            
-            )
-        if self.verbose:
-                print "Done."
-
-        if os.path.exists(os.path.join( self.root_dir, '.bucket3', 'themes', self.theme, 'robots.txt')):
-            if self.verbose:
-                print "   Copying robots.txt...",
-            shutil.copy2(
-                os.path.join(self.root_dir, '.bucket3', 'themes', self.theme, 'robots.txt'),
-                self.html_dir
-                )
-            if self.verbose:
-                print "Done."
-
+    def render_static_pages(self):
         """
         "static" pages are pages that are not blog posts. Their URL is /<page>.html and not YYYY/MM/DD/.../index.html
         Also, they are not included in the RSS feed, in the archive pages, etc.
@@ -475,33 +518,36 @@ class Bucket3():
         {% endblock %}
 
         """
-        if os.path.exists(os.path.join(self.root_dir, 'static')):
-            for static_page in os.listdir(os.path.join(self.root_dir, 'static')):
-                if self.verbose:
-                    print "   Rendering static page %s..." % static_page,
-                tpl = self.tpl_env.get_template(static_page)
-                html = tpl.render()
-                f = open(os.path.join(self.html_dir, static_page), 'w')
-                f.write(html.encode('utf8'))
-                f.close()
-                if self.verbose:
-                    print "Done."
+        print('Rendering static pages (under skel/). ', end='')
+        static_root_src = os.path.join(self.root_dir, 'skel')
+        static_root_dst = self.html_dir
+
+        if os.path.exists(static_root_src):
+            for path, subdirs, files in os.walk(static_root_src):
+                for fname in files:
+                    rel_path = os.path.relpath(path, static_root_src)
+                    dst_path = os.path.join(static_root_dst, rel_path)
+                    ext = os.path.splitext(fname)[1]
+                    
+                    if not os.path.exists(dst_path):
+                        os.makedirs(dst_path)
+                    sys.stdout.write('.')
+                    if ext in ('.html', '.htm'):
+                        html_src = open(os.path.join(path, fname), 'r').read()
+                        html = self.tpl_env.from_string(html_src).render()
+
+                        f = open(os.path.join(dst_path, fname), 'w')
+                        f.write(html)
+                        f.close()
+                    else:
+                        shutil.copy2(os.path.join(path, fname), dst_path)
+        print("Done.")
 
-        # Copy cached images (avatars, etc) from mentions/images to html/images
-        images_src_dir = os.path.join(self.mentions_dir, 'images')
-        images_dst_dir = os.path.join(self.html_dir, 'images')
-        if os.path.exists(images_src_dir):
-            if not os.path.exists(images_dst_dir):
-                os.makedirs(images_dst_dir)
-            for img in os.listdir(images_src_dir):
-                shutil.copy2(os.path.join(images_src_dir, img), images_dst_dir)
 
     def render_post(self, post_id):
         post = self.db_post_get(post_id)
-        mentions = self.mentions_get(post['url'])
-
         tpl = self.tpl_env.get_template('post.html')
-        html = tpl.render(meta=post['meta'], body=post['html'], mentions=mentions)
+        html = tpl.render(meta=post['meta'], body=post['html'])
         
         self.util_write_html(post['meta']['fs_path'],html)
         
@@ -509,14 +555,6 @@ class Bucket3():
             for a in post['meta']['attached']:
                 shutil.copy2(os.path.join(os.path.dirname(self.util_abs_path(post['src'])), a), post['meta']['fs_path'])
 
-        if not os.path.exists(os.path.join(self.root_dir, 'log')):
-            os.makedirs(os.path.join(self.root_dir, 'log'))
-        urls_file = open(os.path.join(self.root_dir, 'log', 'external_links.txt'),'a')
-        for link in self.util_extract_links(post['html']):
-            urls_file.write('%s\t%s\n' % (post['url'], link))
-        urls_file.close()
-
-
     def render_home(self):
         if self.posts_in_homepage:
             count = self.posts_in_homepage
@@ -536,25 +574,33 @@ class Bucket3():
         tpl = self.tpl_env.get_template('sitemap.xml')
         html = tpl.render(posts=posts)
         f = open(os.path.join(self.html_dir, 'sitemap.xml'), 'w')
-        f.write(html.encode('utf8'))
+        f.write(html)
         f.close()
 
     def render_rss(self):
-        print "   rss.xml...",
+        self.render_rss_core('rss.xml')
+        self.render_rss_core('rss-medium.xml')
+        self.render_rss_core('rss-fb.xml')
+        self.render_rss_tags()
+        return
+
+    def render_rss_core(self, template_file='rss.xml'):
+        print("   " + template_file +"...", end='')
         posts = [p for p in self.db_post_get_all(0, 25)]
         if not posts:
             return
-        tpl = self.tpl_env.get_template('rss.xml')
+        tpl = self.tpl_env.get_template(template_file)
         html = tpl.render(posts=posts)
-        f = open(os.path.join(self.html_dir, 'rss.xml'), 'w')
-        f.write(html.encode('utf8'))
+        f = open(os.path.join(self.html_dir, template_file), 'w')
+        f.write(html)
         f.close()
-        print "Done."
+        print("Done.")
 
+    def render_rss_tags(self):
         if not self.rss_tags:
             return
         for tag in self.rss_tags:
-            print "   tag/%s/rss.xml..." % tag,
+            print("   tag/%s/rss.xml..." % tag, end='')
             posts = [p for p in self.db_post_get_by_tag(tag)]
             if posts:
                 tpl = self.tpl_env.get_template('rss.xml')
@@ -563,15 +609,16 @@ class Bucket3():
                 if not os.path.exists(file_dir):
                     os.makedirs(file_dir)
                 f = open(os.path.join(file_dir, 'rss.xml'), 'w')
-                f.write(html.encode('utf8'))
+                f.write(html)
                 f.close()
-            print 'Done.'
+            print('Done.')
+
 
     def render_archive_main(self):
-        counts_by_year_month = self.db_post_get_counts_by_year()
-        if counts_by_year_month:
-            tpl = self.tpl_env.get_template('main_archive.html')
-            html = tpl.render(counts=counts_by_year_month)
+        posts = [p for p in self.db_post_get_all(count=None)]
+        if posts:
+            tpl = self.tpl_env.get_template('archive.html')
+            html = tpl.render(index=posts)
             file_dir = os.path.join(self.html_dir, 'archive')
             self.util_write_html(file_dir, html)
             
@@ -600,15 +647,5 @@ class Bucket3():
             file_dir = os.path.join(self.html_dir, 'tag', tag)
             self.util_write_html(file_dir, html)
 
-    def mentions_get(self, url):
-        url_hash = hashlib.md5(url).hexdigest()
-        path = os.path.join(self.mentions_dir, '%s.yaml' % url_hash)
-        if os.path.isfile(path):
-            f = open(path, mode='r')
-            data = yaml.load(f.read())
-            f.close()
-            return data
-        else:
-            return None
 if __name__ == '__main__':
     pass
